import 'dart:async';

import 'package:audioplayers/audioplayers.dart';
import 'package:flutter/material.dart';

//ignore: must_be_immutable
class AudioViewer extends StatefulWidget {
  final String path;
  final ImageProvider image;
  final String songName;
  final String artistName;
  void Function()? onPlayTap;
  final String? file;
  final BoxDecoration? backgroundStyle;
  final EdgeInsets? screenPadding;
  final double? imageWidth;
  final BorderRadius? imageBorder;
  final BoxFit? imageFit;
  final TextStyle? songNameTextstyle;
  final TextStyle? artistNameTextstyle;
  final Alignment? contentAlignment;
  final SliderThemeData? sliderTheme;
  final Icon? icon;
  final ButtonStyle? iconStyle;
  final void Function()? onPressed;

  AudioViewer(
      {Key? key,
      required this.path,
      required this.image,
      required this.songName,
      required this.artistName,
      this.file,
      this.onPlayTap,
      this.backgroundStyle,
      this.screenPadding,
      this.imageBorder,
      this.imageWidth,
      this.imageFit,
      this.artistNameTextstyle,
      this.songNameTextstyle,
      this.contentAlignment,
      this.sliderTheme,
      this.icon,
      this.iconStyle,
      this.onPressed})
      : super(key: key);

  @override
  State<AudioViewer> createState() => _AudioViewerState();
}

class _AudioViewerState extends State<AudioViewer> {
  late ScrollController _titleController;
  late ScrollController _artistController;
  late AudioPlayer player;
  late Timer audioTimestamp;
  double sliderValue = 0;
  Duration audioDuration = const Duration(seconds: 1000);
  Duration currentPosition = Duration.zero;

  @override
  void initState() {
    super.initState();
    _titleController = ScrollController();
    _artistController = ScrollController();
    player = AudioPlayer();
    _startAutoScroll(_titleController);
    _startAutoScroll(_artistController);
    playAudio();
    startAudioTime();
  }

  @override
  void dispose() {
    _titleController.dispose();
    _artistController.dispose();
    audioTimestamp.cancel();
    player.dispose();
    super.dispose();
  }

  void _startAutoScroll(ScrollController controller) {
    Timer.periodic(const Duration(seconds: 10), (_) {
      if (controller.hasClients) {
        double maxScrollExtent = controller.position.maxScrollExtent;
        double minScrollExtent = controller.position.minScrollExtent;
        if (controller.offset != maxScrollExtent) {
          controller.animateTo(maxScrollExtent, duration: const Duration(seconds: 5), curve: Curves.linear);
        } else {
          controller.animateTo(minScrollExtent, duration: const Duration(seconds: 5), curve: Curves.linear);
        }
      }
    });
  }

  void playAudio() {
    switch (widget.file) {
      case 'url':
        player.play(UrlSource(widget.path));
        break;
      case 'assets':
        player.play(AssetSource(widget.path));
        break;
      case 'local':
        player.play(DeviceFileSource(widget.path));
        break;
      default:
        null;
    }
  }

  void resumeAudio() {
    player.resume();
  }

  void pauseAudio() {
    player.pause();
  }

  void startAudioTime() {
    audioTimestamp = Timer.periodic(const Duration(seconds: 1), (timer) async {
      Duration? currentTimestamp = await player.getCurrentPosition();
      audioDuration = await player.getDuration() ?? const Duration(seconds: 1000);
      if ((currentTimestamp?.inSeconds ?? 0) < audioDuration.inSeconds) {
        setState(() {
          sliderValue = currentTimestamp?.inSeconds.toDouble() ?? 0;
          currentPosition = currentTimestamp ?? Duration.zero;
        });
      } else {
        timer.cancel();
        sliderValue = audioDuration.inSeconds.toDouble();
        currentPosition = audioDuration;
      }
    });
  }

  void seekTo(double value) {
    setState(() {
      player.seek(Duration(seconds: value.toInt()));
      sliderValue = value;
    });
  }

  String formatDuration(Duration duration) {
    String twoDigits(int n) => n.toString().padLeft(2, '0');
    String minutes = duration.inMinutes.remainder(60).toString();
    String twoDigitSeconds = twoDigits(duration.inSeconds.remainder(60));
    return '$minutes:$twoDigitSeconds';
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        body: Container(
            padding: widget.screenPadding ?? const EdgeInsets.all(36),
            decoration: widget.backgroundStyle ??
                const BoxDecoration(
                    gradient: LinearGradient(
                        begin: Alignment.topRight,
                        end: Alignment.bottomLeft,
                        colors: [Color.fromRGBO(255, 250, 158, 1), Color.fromRGBO(119, 140, 69, 1)])),
            child: Column(children: [
              const SizedBox(height: kToolbarHeight),
              SizedBox(
                  width: widget.imageWidth ?? MediaQuery.of(context).size.width,
                  child: ClipRRect(
                      borderRadius: widget.imageBorder ?? BorderRadius.circular(10),
                      child: Image(image: widget.image, fit: widget.imageFit ?? BoxFit.fitWidth))),
              const SizedBox(height: 30),
              Align(
                  alignment: widget.contentAlignment ?? Alignment.topLeft,
                  child: Column(children: [
                    SingleChildScrollView(
                        scrollDirection: Axis.horizontal,
                        controller: _titleController,
                        child: Text(widget.songName,
                            style: widget.songNameTextstyle ??
                                const TextStyle(fontSize: 26, fontWeight: FontWeight.bold, color: Colors.white))),
                    SingleChildScrollView(
                        scrollDirection: Axis.horizontal,
                        controller: _artistController,
                        child: Text(widget.artistName,
                            style: widget.artistNameTextstyle ??
                                const TextStyle(fontSize: 20, fontWeight: FontWeight.w600, color: Colors.white70))),
                    const SizedBox(height: 10),
                    SliderTheme(
                        data: widget.sliderTheme ??
                            SliderThemeData(
                                overlayShape: SliderComponentShape.noOverlay,
                                thumbColor: Colors.white,
                                activeTrackColor: Colors.white,
                                inactiveTrackColor: Colors.black26),
                        child: Slider(
                            value: sliderValue,
                            min: 0,
                            max: audioDuration.inSeconds.toDouble(),
                            onChanged: (value) {
                              setState(() {
                                sliderValue = value;
                              });
                            },
                            onChangeEnd: (value) {
                              seekTo(value);
                            })),
                    Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: [
                        Text(formatDuration(currentPosition),
                            style: const TextStyle(fontSize: 12, fontWeight: FontWeight.bold, color: Colors.white)),
                        Text(formatDuration(audioDuration - currentPosition),
                            style: const TextStyle(fontSize: 12, fontWeight: FontWeight.bold, color: Colors.white))
                      ],
                    )
                  ])),
              const SizedBox(height: 20),
              IconButton.filled(
                  onPressed: widget.onPressed ??
                      () {
                        setState(() {
                          (player.state.name == 'playing') ? player.pause() : player.resume();
                        });
                      },
                  icon: widget.icon ?? Icon(player.state.name == 'playing' ? Icons.pause : Icons.play_arrow),
                  style: widget.iconStyle ??
                      const ButtonStyle(
                          backgroundColor: MaterialStatePropertyAll<Color>(Colors.white),
                          iconColor: MaterialStatePropertyAll<Color>(Colors.black)))
            ])));
  }
}
