import 'dart:async';

import 'package:flutter/material.dart';
import 'package:video_player/video_player.dart';

class VideoViewer extends StatefulWidget {
  const VideoViewer(
      {super.key,
      required this.uri,
      this.iconColor,
      this.iconSize,
      this.indicatorHeight,
      this.onPressed,
      this.style,
      this.icon,
      this.loadingIconColor,
      this.sliderStyle,
      this.timeStampStyle,
      this.onChangeEnd,
      this.onSliderChange,
      this.videoAspectRatio,
      this.onVideoScreenTap});

  final String uri;
  final Color? iconColor;
  final double? iconSize;
  final double? indicatorHeight;
  final void Function()? onPressed;
  final ButtonStyle? style;
  final Color? loadingIconColor;
  final Icon? icon;
  final SliderThemeData? sliderStyle;
  final TextStyle? timeStampStyle;
  final Function(double)? onSliderChange;
  final Function(double)? onChangeEnd;
  final double? videoAspectRatio;
  final void Function()? onVideoScreenTap;

  @override
  State<VideoViewer> createState() => _VideoViewerState();
}

class _VideoViewerState extends State<VideoViewer> {
  late VideoPlayerController controller;
  late Future<void> initializeVideoPlayerFuture;
  late Timer videoTimestamp;
  Timer disableShowIconTimer = Timer.periodic((const Duration(seconds: 5)), (timer) => {});
  bool isShowIcon = false;
  double sliderValue = 0;
  Duration currentPosition = Duration.zero;

  @override
  void initState() {
    super.initState();
    controller = VideoPlayerController.networkUrl(
      Uri.parse(widget.uri),
    );
    initializeVideoPlayerFuture = controller.initialize();
    playVideo();
    startTimer();
  }

  @override
  void dispose() {
    controller.dispose();
    disableShowIconTimer.cancel();
    videoTimestamp.cancel();
    super.dispose();
  }

  void startTimer() {
    videoTimestamp = Timer.periodic(const Duration(seconds: 1), (timer) {
      if (controller.value.position.inSeconds < controller.value.duration.inSeconds) {
        setState(() {
          sliderValue = controller.value.position.inSeconds.toDouble();
          currentPosition = controller.value.position;
        });
      } else {
        onVideoCompleted();
        currentPosition = controller.value.duration;
        timer.cancel();
      }
    });
  }

  void onVideoCompleted() {
    setState(() {
      sliderValue = controller.value.duration.inSeconds.toDouble();
    });
  }

  void _seekTo(double value) {
    setState(() {
      controller.seekTo(Duration(seconds: value.toInt()));
      sliderValue = value;
    });
  }

  void playVideo() {
    controller.play();
  }

  void pauseVideo(){
    controller.pause();
  }

  String formatDuration(Duration duration) {
    String twoDigits(int n) => n.toString().padLeft(2, '0');
    String minutes = duration.inMinutes.remainder(60).toString();
    String twoDigitSeconds = twoDigits(duration.inSeconds.remainder(60));
    return '$minutes:$twoDigitSeconds';
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        body: FutureBuilder(
            future: initializeVideoPlayerFuture,
            builder: (context, snapshot) {
              return snapshot.connectionState == ConnectionState.done
                  ? AspectRatio(
                      aspectRatio: widget.videoAspectRatio ?? controller.value.aspectRatio,
                      child: Stack(children: [
                        GestureDetector(
                            onTap: widget.onVideoScreenTap ?? () {
                              setState(() {
                                isShowIcon = !isShowIcon;
                                disableShowIconTimer = Timer.periodic(
                                    (const Duration(seconds: 5)),
                                    (timer) => setState(() {
                                          isShowIcon = false;
                                        }));
                              });
                            },
                            child: Stack(children: [
                              VideoPlayer(controller),
                              Visibility(
                                  visible: isShowIcon,
                                  child: Stack(children: [
                                    Container(color: Colors.black.withOpacity(0.6)),
                                    Center(
                                        child: IconButton(
                                            icon: widget.icon ??
                                                Icon(controller.value.isPlaying ? Icons.pause : Icons.play_arrow),
                                            iconSize: widget.iconSize ?? 50,
                                            color: widget.iconColor ?? Colors.white,
                                            onPressed: widget.onPressed ??
                                                () {
                                                  setState(() {
                                                    if (controller.value.isPlaying) {
                                                      controller.pause();
                                                    } else {
                                                      controller.play();
                                                    }
                                                  });
                                                }))
                                  ]))
                            ])),
                        Positioned(
                            bottom: 0,
                            width: MediaQuery.of(context).size.width,
                            child: Visibility(
                                visible: isShowIcon,
                                child: Column(crossAxisAlignment: CrossAxisAlignment.start, children: [
                                  Padding(
                                      padding: const EdgeInsets.only(left: 8),
                                      child: Text(
                                          '${formatDuration(currentPosition)} / ${formatDuration(controller.value.duration)}',
                                          style: widget.timeStampStyle ?? const TextStyle(color: Colors.white, fontWeight: FontWeight.bold))),
                                  SliderTheme(
                                      data: widget.sliderStyle ?? SliderThemeData(
                                          overlayShape: SliderComponentShape.noOverlay,
                                          activeTrackColor: Colors.white,
                                          inactiveTrackColor: Colors.grey.withOpacity(0.5),
                                          thumbColor: Colors.white),
                                      child: Slider(
                                          value: sliderValue,
                                          min: 0,
                                          max: controller.value.duration.inSeconds.toDouble(),
                                          onChanged: widget.onSliderChange ?? (value) {
                                            setState(() {
                                              sliderValue = value;
                                            });
                                          },
                                          onChangeEnd: widget.onChangeEnd ?? (value) {
                                            _seekTo(value);
                                          }))
                                ])))
                      ]))
                  : Container(
                      height: widget.indicatorHeight,
                      color: Colors.grey.withOpacity(0.4),
                      child: Center(
                          child: SizedBox(
                              width: MediaQuery.of(context).size.width / 10,
                              child: CircularProgressIndicator(
                                  color: widget.loadingIconColor ?? Colors.white.withOpacity(0.6)))));
            }));
  }
}
